@@ -1,4 +1,4 @@
-#! /bin/sh
+#!/bin/bash
 #
 # ***** BEGIN LICENSE BLOCK *****
 # Version: MPL 1.1/GPL 2.0/LGPL 2.1
@@ -17,7 +17,7 @@
 #
 # The Initial Developer of the Original Code is
 # Sun Microsystems, Inc.
-# Portions created by the Initial Developer are Copyright (C) 2006-2007
+# Portions created by the Initial Developer are Copyright (C) 2006-2009
 # the Initial Developer. All Rights Reserved.
 #
 # Contributor(s):
@@ -64,7 +64,7 @@ memleak_init()
 	fi
 	
 	if [ ! -r ${CERT_LOG_FILE} ]; then
-		cd ../cert
+		cd ${QADIR}/cert
 		. ./cert.sh
 	fi
 
@@ -73,17 +73,18 @@ memleak_init()
 		CLEANUP="${SCRIPTNAME}"
 	fi
 
- 	NSS_DISABLE_ARENA_FREE_LIST="1"
- 	export NSS_DISABLE_ARENA_FREE_LIST
-	
 	OLD_LIBRARY_PATH=${LD_LIBRARY_PATH}
-	TMP_LIBDIR="${HOSTDIR}/tmp$$"
-	TMP_STACKS="${HOSTDIR}/stacks$$"
-	TMP_COUNT="${HOSTDIR}/count$$"
+	TMP_LIBDIR="${HOSTDIR}/tmp"
+	TMP_STACKS="${HOSTDIR}/stacks"
+	TMP_SORTED="${HOSTDIR}/sorted"
+	TMP_COUNT="${HOSTDIR}/count"
+	DBXOUT="${HOSTDIR}/dbxout"
+	DBXERR="${HOSTDIR}/dbxerr"
+	DBXCMD="${HOSTDIR}/dbxcmd"
 	
 	PORT=${PORT:-8443}
 	
-	MODE_LIST="NORMAL BYPASS FIPS"	
+	MODE_LIST="NORMAL BYPASS FIPS"
 	
 	SERVER_DB="${HOSTDIR}/server_memleak"
 	CLIENT_DB="${HOSTDIR}/client_memleak"
@@ -137,8 +138,7 @@ memleak_init()
 			FREEBL_LIST="${FREEBL_DEFAULT}"
 		fi
 		
-		RUN_SELFSERV_DBG="run_selfserv_dbx"
-		RUN_STRSCLNT_DBG="run_strsclnt_dbx"
+		RUN_COMMAND_DBG="run_command_dbx"
 		PARSE_LOGFILE="parse_logfile_dbx"
 		;;
 	"Linux")
@@ -160,8 +160,7 @@ memleak_init()
 		FREEBL_DEFAULT="libfreebl_3"
 		FREEBL_LIST="${FREEBL_DEFAULT}"
 				
-		RUN_SELFSERV_DBG="run_selfserv_valgrind"
-		RUN_STRSCLNT_DBG="run_strsclnt_valgrind"
+		RUN_COMMAND_DBG="run_command_valgrind"
 		PARSE_LOGFILE="parse_logfile_valgrind"
 		;;
 	*)
@@ -170,7 +169,7 @@ memleak_init()
 		;;
 	esac
 
-	if [ "${BUILD_OPT}" -eq 1 ] ; then
+	if [ "${BUILD_OPT}" = "1" ] ; then
 		OPT="OPT"
 	else 
 		OPT="DBG"
@@ -180,11 +179,15 @@ memleak_init()
 	export NSS_DISABLE_UNLOAD
 
 	SELFSERV_ATTR="-D -p ${PORT} -d ${SERVER_DB} -n ${HOSTADDR} -e ${HOSTADDR}-ec -w nss -c ABCDEF:C001:C002:C003:C004:C005:C006:C007:C008:C009:C00A:C00B:C00C:C00D:C00E:C00F:C010:C011:C012:C013:C014cdefgijklmnvyz -t 5"
-	TSTCLNT_ATTR="-p ${PORT} -h ${HOSTADDR} -c j -f -d ${CLIENT_DB} -w nss"
+	TSTCLNT_ATTR="-p ${PORT} -h ${HOSTADDR} -c j -f -d ${CLIENT_DB} -w nss -o"
+	STRSCLNT_ATTR="-q -p ${PORT} -d ${CLIENT_DB} -w nss -c 1000 -n TestUser ${HOSTADDR}"
 
 	tbytes=0
 	tblocks=0
 	truns=0
+	
+	MEMLEAK_DBG=1
+	export MEMLEAK_DBG
 }
 
 ########################### memleak_cleanup ############################
@@ -192,6 +195,7 @@ memleak_init()
 ########################################################################
 memleak_cleanup()
 {
+	unset MEMLEAK_DBG
 	unset NSS_DISABLE_UNLOAD
 	
 	. ${QADIR}/common/cleanup.sh
@@ -217,27 +221,27 @@ set_test_mode()
 	fi
 	
 	if [ "${server_mode}" = "FIPS" ] ; then
-		modutil -dbdir ${SERVER_DB} -fips true -force
-		modutil -dbdir ${SERVER_DB} -list
-		modutil -dbdir ${CLIENT_DB} -fips false -force
-		modutil -dbdir ${CLIENT_DB} -list
+		${BINDIR}/modutil -dbdir ${SERVER_DB} -fips true -force
+		${BINDIR}/modutil -dbdir ${SERVER_DB} -list
+		${BINDIR}/modutil -dbdir ${CLIENT_DB} -fips false -force
+		${BINDIR}/modutil -dbdir ${CLIENT_DB} -list
 		
 		echo "${SCRIPTNAME}: FIPS is ON"
 		cipher_list="c d e i j k n v y z"
 	elif [ "${client_mode}" = "FIPS" ] ; then
 		
-		modutil -dbdir ${SERVER_DB} -fips false -force
-		modutil -dbdir ${SERVER_DB} -list
-		modutil -dbdir ${CLIENT_DB} -fips true -force
-		modutil -dbdir ${CLIENT_DB} -list
+		${BINDIR}/modutil -dbdir ${SERVER_DB} -fips false -force
+		${BINDIR}/modutil -dbdir ${SERVER_DB} -list
+		${BINDIR}/modutil -dbdir ${CLIENT_DB} -fips true -force
+		${BINDIR}/modutil -dbdir ${CLIENT_DB} -list
 		
 		echo "${SCRIPTNAME}: FIPS is ON"
 		cipher_list="c d e i j k n v y z"
 	else
-		modutil -dbdir ${SERVER_DB} -fips false -force
-		modutil -dbdir ${SERVER_DB} -list
-		modutil -dbdir ${CLIENT_DB} -fips false -force
-		modutil -dbdir ${CLIENT_DB} -list
+		${BINDIR}/modutil -dbdir ${SERVER_DB} -fips false -force
+		${BINDIR}/modutil -dbdir ${SERVER_DB} -list
+		${BINDIR}/modutil -dbdir ${CLIENT_DB} -fips false -force
+		${BINDIR}/modutil -dbdir ${CLIENT_DB} -list
 		
 		echo "${SCRIPTNAME}: FIPS is OFF"
 		cipher_list="A B C D E F :C001 :C002 :C003 :C004 :C005 :C006 :C007 :C008 :C009 :C00A :C010 :C011 :C012 :C013 :C014 c d e f g i j k l m n v y z"
@@ -256,18 +260,35 @@ set_freebl()
 		if [ -d "${TMP_LIBDIR}" ] ; then
 			rm -rf ${TMP_LIBDIR}
 		fi
+
 		mkdir ${TMP_LIBDIR}
+		[ $? -ne 0 ] && html_failed "Create temp directory" && return 1
+
 		cp ${DIST}/${OBJDIR}/lib/*.so ${DIST}/${OBJDIR}/lib/*.chk ${TMP_LIBDIR}
+		[ $? -ne 0 ] && html_failed "Copy libraries to temp directory" && return 1
 		
 		echo "${SCRIPTNAME}: Using ${freebl} instead of ${FREEBL_DEFAULT}"
+
 		mv ${TMP_LIBDIR}/${FREEBL_DEFAULT}.so ${TMP_LIBDIR}/${FREEBL_DEFAULT}.so.orig
+		[ $? -ne 0 ] && html_failed "Move ${FREEBL_DEFAULT}.so -> ${FREEBL_DEFAULT}.so.orig" && return 1
+
 		cp ${TMP_LIBDIR}/${freebl}.so ${TMP_LIBDIR}/${FREEBL_DEFAULT}.so
+		[ $? -ne 0 ] && html_failed "Copy ${freebl}.so -> ${FREEBL_DEFAULT}.so" && return 1
+
 		mv ${TMP_LIBDIR}/${FREEBL_DEFAULT}.chk ${TMP_LIBDIR}/${FREEBL_DEFAULT}.chk.orig
+		[ $? -ne 0 ] && html_failed "Move ${FREEBL_DEFAULT}.chk -> ${FREEBL_DEFAULT}.chk.orig" && return 1
+
 		cp ${TMP_LIBDIR}/${freebl}.chk ${TMP_LIBDIR}/${FREEBL_DEFAULT}.chk
-		
+		[ $? -ne 0 ] && html_failed "Copy ${freebl}.chk to temp directory" && return 1
+
+		echo "ls -l ${TMP_LIBDIR}"
+		ls -l ${TMP_LIBDIR}
+
 		LD_LIBRARY_PATH="${TMP_LIBDIR}"
 		export LD_LIBRARY_PATH
 	fi
+
+	return 0
 }
 
 ############################# clear_freebl #############################
@@ -284,6 +305,64 @@ clear_freebl()
 	fi
 }
 
+############################ run_command_dbx ###########################
+# local shell function to run command under dbx tool
+########################################################################
+run_command_dbx()
+{
+	COMMAND=$1
+	shift
+	ATTR=$*
+	
+	COMMAND=`which ${COMMAND}`
+	
+	echo "dbxenv follow_fork_mode parent" > ${DBXCMD}
+	echo "dbxenv rtc_mel_at_exit verbose" >> ${DBXCMD}
+	echo "dbxenv rtc_biu_at_exit verbose" >> ${DBXCMD}
+	echo "check -memuse -match 16 -frames 16" >> ${DBXCMD}
+	echo "run ${ATTR}" >> ${DBXCMD}
+	
+	export NSS_DISABLE_ARENA_FREE_LIST=1
+	
+	echo "${SCRIPTNAME}: -------- Running ${COMMAND} under DBX:"
+	echo "${DBX} ${COMMAND}"
+	echo "${SCRIPTNAME}: -------- DBX commands:"
+	cat ${DBXCMD}
+	
+	( ${DBX} ${COMMAND} < ${DBXCMD} > ${DBXOUT} 2> ${DBXERR} )
+	grep -v Reading ${DBXOUT} 1>&2
+	cat ${DBXERR}
+	
+	unset NSS_DISABLE_ARENA_FREE_LIST
+	
+	grep "exit code is" ${DBXOUT}
+	grep "exit code is 0" ${DBXOUT} > /dev/null
+	return $?
+}
+
+######################### run_command_valgrind #########################
+# local shell function to run command under valgrind tool
+########################################################################
+run_command_valgrind()
+{
+	COMMAND=$1
+	shift
+	ATTR=$*
+	
+	export NSS_DISABLE_ARENA_FREE_LIST=1
+	
+	echo "${SCRIPTNAME}: -------- Running ${COMMAND} under Valgrind:"
+	echo "${VALGRIND} --tool=memcheck --leak-check=yes --show-reachable=yes --partial-loads-ok=yes --leak-resolution=high --num-callers=50 ${COMMAND} ${ATTR}"
+	echo "Running: ${COMMAND} ${ATTR}" 1>&2
+	${VALGRIND} --tool=memcheck --leak-check=yes --show-reachable=yes --partial-loads-ok=yes --leak-resolution=high --num-callers=50 ${COMMAND} ${ATTR} 1>&2
+	ret=$?
+	echo "==0=="
+	
+	unset NSS_DISABLE_ARENA_FREE_LIST
+	
+	return $ret
+}
+
 ############################# run_selfserv #############################
 # local shell function to start selfserv
 ########################################################################
@@ -293,53 +372,29 @@ run_selfserv()
 	echo "LD_LIBRARY_PATH=${LD_LIBRARY_PATH}"
 	echo "${SCRIPTNAME}: -------- Running selfserv:"
 	echo "selfserv ${SELFSERV_ATTR}"
-	selfserv ${SELFSERV_ATTR}
+	${BINDIR}/selfserv ${SELFSERV_ATTR}
+	ret=$?
+	if [ $ret -ne 0 ]; then
+		html_failed "${LOGNAME}: Selfserv"
+		echo "${SCRIPTNAME} ${LOGNAME}: " \
+			"Selfserv produced a returncode of ${ret} - FAILED"
+	fi
 }
 
-########################### run_selfserv_dbx ###########################
-# local shell function to start selfserv under dbx tool
+########################### run_selfserv_dbg ###########################
+# local shell function to start selfserv under debug tool
 ########################################################################
-run_selfserv_dbx()
+run_selfserv_dbg()
 {
-	DBX_CMD="${HOSTDIR}/run_selfserv$$.dbx"
-	
-	cat << EOF_DBX > ${DBX_CMD}
-dbxenv follow_fork_mode parent
-dbxenv rtc_mel_at_exit verbose
-dbxenv rtc_biu_at_exit verbose
-check -memuse -match 16 -frames 16
-run ${SERVER_OPTION} ${SELFSERV_ATTR}
-EOF_DBX
-	
-	SELFSERV=`which selfserv`
 	echo "PATH=${PATH}"
 	echo "LD_LIBRARY_PATH=${LD_LIBRARY_PATH}"
-	echo "${SCRIPTNAME}: -------- Running selfserv under DBX:"
-	echo "${DBX} ${SELFSERV} < ${DBX_CMD}"
-	echo "${SCRIPTNAME}: -------- DBX commands (${DBX_CMD}):"
-	cat ${DBX_CMD}
-	${DBX} ${SELFSERV} < ${DBX_CMD} 2>/dev/null | grep -v Reading
-	rm ${DBX_CMD}
-}
-
-######################### run_selfserv_valgrind ########################
-# local shell function to start selfserv under valgrind tool
-########################################################################
-run_selfserv_valgrind()
-{	
-	echo "PATH=${PATH}"
-	echo "LD_LIBRARY_PATH=${LD_LIBRARY_PATH}"
-	echo "${SCRIPTNAME}: -------- Running selfserv under Valgrind:"
-	echo "${VALGRIND} --tool=memcheck --leak-check=yes --show-reachable=yes --partial-loads-ok=yes --leak-resolution=high --num-callers=50 selfserv ${SELFSERV_ATTR}"
-	${VALGRIND} --tool=memcheck --leak-check=yes --show-reachable=yes --partial-loads-ok=yes --leak-resolution=high --num-callers=50 selfserv ${SELFSERV_ATTR}
-}
-
-############################ strsclnt_attr #############################
-# local shell function to set strsclnt attributes and parameters
-########################################################################
-strsclnt_attr()
-{
-	STRSCLNT_ATTR="-q -p ${PORT} -d ${CLIENT_DB} -w nss -c 1000 -C ${cipher} ${HOSTADDR}"
+	${RUN_COMMAND_DBG} ${BINDIR}/selfserv ${SERVER_OPTION} ${SELFSERV_ATTR}
+	ret=$?
+	if [ $ret -ne 0 ]; then
+		html_failed "${LOGNAME}: Selfserv"
+		echo "${SCRIPTNAME} ${LOGNAME}: " \
+			"Selfserv produced a returncode of ${ret} - FAILED"
+	fi
 }
 
 ############################# run_strsclnt #############################
@@ -349,74 +404,117 @@ strsclnt_attr()
 run_strsclnt()
 {
 	for cipher in ${cipher_list}; do
-		strsclnt_attr ${cipher}
+		ATTR="${STRSCLNT_ATTR} -C ${cipher}"
 		echo "${SCRIPTNAME}: -------- Trying cipher ${cipher}:"
-		echo "strsclnt ${STRSCLNT_ATTR}"
-		strsclnt ${STRSCLNT_ATTR}
+		echo "strsclnt ${ATTR}"
+		${BINDIR}/strsclnt ${ATTR}
+		ret=$?
+		if [ $ret -ne 0 ]; then
+			html_failed "${LOGNAME}: Strsclnt with cipher ${cipher}"
+			echo "${SCRIPTNAME} ${LOGNAME}: " \
+				"Strsclnt produced a returncode of ${ret} - FAILED"
+		fi
 	done
 	
-	echo "PATH=${PATH}"
-	echo "LD_PATH=${LD_LIBRARY_PATH}"
-
-	echo "${SCRIPTNAME}: -------- Stopping server:"
-	echo "tstclnt ${TSTCLNT_ATTR} < ${REQUEST_FILE}"
-	tstclnt ${TSTCLNT_ATTR} < ${REQUEST_FILE}
-}
-
-########################### run_strsclnt_dbx ###########################
-# local shell function to run strsclnt under dbx for all ciphers and 
-# send stop command to selfserv over tstclnt
-########################################################################
-run_strsclnt_dbx()
-{
-	DBX_CMD="${HOSTDIR}/run_strsclnt$$.dbx"
-		
-	for cipher in ${cipher_list}; do
-		strsclnt_attr ${cipher}
-		cat << EOF_DBX > ${DBX_CMD}
-dbxenv follow_fork_mode parent
-dbxenv rtc_mel_at_exit verbose
-dbxenv rtc_biu_at_exit verbose
-check -memuse -match 16 -frames 16
-run ${CLIENT_OPTION} ${STRSCLNT_ATTR}
-EOF_DBX
-
-		STRSCLNT=`which strsclnt`
-		echo "${SCRIPTNAME}: -------- Trying cipher ${cipher} under DBX:"
-		echo "${DBX} ${STRSCLNT} < ${DBX_CMD}"
-		echo "${SCRIPTNAME}: -------- DBX comands (${DBX_CMD}):"
-		cat ${DBX_CMD}
-		${DBX} ${STRSCLNT} < ${DBX_CMD} 2>/dev/null | grep -v Reading
-	done
-	
-	echo "PATH=${PATH}"
-	echo "LD_PATH=${LD_LIBRARY_PATH}"
-	
 	echo "${SCRIPTNAME}: -------- Stopping server:"
 	echo "tstclnt ${TSTCLNT_ATTR} < ${REQUEST_FILE}"
-	tstclnt ${TSTCLNT_ATTR} < ${REQUEST_FILE}
-	rm ${DBX_CMD}
+	${BINDIR}/tstclnt ${TSTCLNT_ATTR} < ${REQUEST_FILE}
+	ret=$?
+	if [ $ret -ne 0 ]; then
+		html_failed "${LOGNAME}: Tstclnt"
+		echo "${SCRIPTNAME} ${LOGNAME}: " \
+			"Tstclnt produced a returncode of ${ret} - FAILED"
+	fi
+	
+	kill $(jobs -p) 2> /dev/null
 }
 
-######################## run_strsclnt_valgrind #########################
-# local shell function to run strsclnt under valgrind for all ciphers 
+########################### run_strsclnt_dbg ###########################
+# local shell function to run strsclnt under debug tool for all ciphers 
 # and send stop command to selfserv over tstclnt
 ########################################################################
-run_strsclnt_valgrind()
+run_strsclnt_dbg()
 {
 	for cipher in ${cipher_list}; do
-		strsclnt_attr ${cipher}
-		echo "${SCRIPTNAME}: -------- Trying cipher ${cipher} under Valgrind:"
-		echo "${VALGRIND} --tool=memcheck --leak-check=yes --show-reachable=yes --partial-loads-ok=yes --leak-resolution=high --num-callers=50 strsclnt ${STRSCLNT_ATTR}"
-		${VALGRIND} --tool=memcheck --leak-check=yes --show-reachable=yes --partial-loads-ok=yes --leak-resolution=high --num-callers=50 strsclnt ${STRSCLNT_ATTR}
+		ATTR="${STRSCLNT_ATTR} -C ${cipher}"
+		${RUN_COMMAND_DBG} ${BINDIR}/strsclnt ${CLIENT_OPTION} ${ATTR}
+		ret=$?
+		if [ $ret -ne 0 ]; then
+			html_failed "${LOGNAME}: Strsclnt with cipher ${cipher}"
+			echo "${SCRIPTNAME} ${LOGNAME}: " \
+				"Strsclnt produced a returncode of ${ret} - FAILED"
+		fi
 	done
-
-	echo "PATH=${PATH}"
-	echo "LD_PATH=${LD_LIBRARY_PATH}"
 	
 	echo "${SCRIPTNAME}: -------- Stopping server:"
 	echo "tstclnt ${TSTCLNT_ATTR} < ${REQUEST_FILE}"
-	tstclnt ${TSTCLNT_ATTR} < ${REQUEST_FILE}
+	${BINDIR}/tstclnt ${TSTCLNT_ATTR} < ${REQUEST_FILE}
+	ret=$?
+	if [ $ret -ne 0 ]; then
+		html_failed "${LOGNAME}: Tstclnt"
+		echo "${SCRIPTNAME} ${LOGNAME}: " \
+			"Tstclnt produced a returncode of ${ret} - FAILED"
+	fi
+	
+	kill $(jobs -p) 2> /dev/null
+}
+
+stat_clear()
+{
+	stat_minbytes=9999999
+	stat_maxbytes=0
+	stat_minblocks=9999999
+	stat_maxblocks=0
+	stat_bytes=0
+	stat_blocks=0
+	stat_runs=0
+}
+
+stat_add()
+{
+	read hash lbytes bytes_str lblocks blocks_str in_str lruns runs_str \
+		minbytes minbytes_str maxbytes maxbytes_str minblocks \
+		minblocks_str maxblocks maxblocks_str rest < ${TMP_COUNT} 
+	rm ${TMP_COUNT}
+	
+	tbytes=`expr ${tbytes} + ${lbytes}`
+	tblocks=`expr ${tblocks} + ${lblocks}`
+	truns=`expr ${truns} + ${lruns}`
+	
+	if [ ${stat_minbytes} -gt ${minbytes} ]; then
+		stat_minbytes=${minbytes}
+	fi
+			
+	if [ ${stat_maxbytes} -lt ${maxbytes} ]; then
+		stat_maxbytes=${maxbytes}
+	fi
+			
+	if [ ${stat_minblocks} -gt ${minblocks} ]; then
+		stat_minblocks=${minblocks}
+	fi
+			
+	if [ ${stat_maxblocks} -lt ${maxblocks} ]; then
+		stat_maxblocks=${maxblocks}
+	fi
+			
+	stat_bytes=`expr ${stat_bytes} + ${lbytes}`
+	stat_blocks=`expr ${stat_blocks} + ${lblocks}`
+	stat_runs=`expr ${stat_runs} + ${lruns}`
+}
+
+stat_print()
+{
+	if [ ${stat_runs} -gt 0 ]; then
+		stat_avgbytes=`expr "${stat_bytes}" / "${stat_runs}"`
+		stat_avgblocks=`expr "${stat_blocks}" / "${stat_runs}"`
+		
+		echo
+		echo "$1 statistics:"
+		echo "Leaked bytes: ${stat_minbytes} min, ${stat_avgbytes} avg, ${stat_maxbytes} max"
+		echo "Leaked blocks: ${stat_minblocks} min, ${stat_avgblocks} avg, ${stat_maxblocks} max"
+		echo "Total runs: ${stat_runs}"
+		echo
+	fi
 }
 
 ########################## run_ciphers_server ##########################
@@ -426,35 +524,37 @@ run_ciphers_server()
 {
 	html_head "Memory leak checking - server"
 	
+	stat_clear
+	
 	client_mode="NORMAL"	
 	for server_mode in ${MODE_LIST}; do
 		set_test_mode
 		
 		for freebl in ${FREEBL_LIST}; do
-			set_freebl
+			set_freebl || continue
 			
 			LOGNAME=server-${BIT_NAME}-${freebl}-${server_mode}
 			LOGFILE=${LOGDIR}/${LOGNAME}.log
 			echo "Running ${LOGNAME}"
 			
-			${RUN_SELFSERV_DBG} 2>&1 | tee ${LOGFILE} &
-			sleep 5
-			run_strsclnt
+			(
+			    run_selfserv_dbg 2>> ${LOGFILE} &
+			    sleep 5
+			    run_strsclnt
+			)
 			
 			sleep 20
 			clear_freebl
 			
-			if [ ${OS_NAME} = "Linux" ]; then
-				echo "==0==" >> ${LOGFILE}
-			fi
-			echo "" >> ${LOGFILE}
-
-			log_parse >> ${FOUNDLEAKS}
+			log_parse
 			ret=$?
+			
 			html_msg ${ret} 0 "${LOGNAME}" "produced a returncode of $ret, expected is 0"
 		done
 	done
 	
+	stat_print "Selfserv"
+	
 	html "</TABLE><BR>"
 }
 
@@ -465,35 +565,36 @@ run_ciphers_client()
 {
 	html_head "Memory leak checking - client"
 	
+	stat_clear
+	
 	server_mode="NORMAL"
 	for client_mode in ${MODE_LIST}; do
 		set_test_mode
 		
 		for freebl in ${FREEBL_LIST}; do
-			set_freebl
+			set_freebl || continue
 			
 			LOGNAME=client-${BIT_NAME}-${freebl}-${client_mode}
 			LOGFILE=${LOGDIR}/${LOGNAME}.log
 			echo "Running ${LOGNAME}"
 			
-			run_selfserv &
-			sleep 5
-			${RUN_STRSCLNT_DBG} 2>&1 | tee ${LOGFILE}
+			(
+			    run_selfserv &
+			    sleep 5
+			    run_strsclnt_dbg 2>> ${LOGFILE}
+			)
 			
 			sleep 20
 			clear_freebl
-
-			if [ ${OS_NAME} = "Linux" ]; then
-				echo "==0==" >> ${LOGFILE}
-			fi
-			echo "" >> ${LOGFILE}
-
-			log_parse >> ${FOUNDLEAKS}
+			
+			log_parse
 			ret=$?
 			html_msg ${ret} 0 "${LOGNAME}" "produced a returncode of $ret, expected is 0"
 		done
 	done
 	
+	stat_print "Strsclnt"
+	
 	html "</TABLE><BR>"
 }
 
@@ -507,7 +608,13 @@ parse_logfile_dbx()
 		in_mel = 0
 		mel_line = 0
 		bytes = 0
+		lbytes = 0
+		minbytes = 9999999
+		maxbytes = 0
 		blocks = 0
+		lblocks = 0
+		minblocks = 9999999
+		maxblocks = 0
 		runs = 0
 		stack_string = ""
 		bin_name = ""
@@ -516,7 +623,7 @@ parse_logfile_dbx()
 	/Possible memory leak -- address in block \(aib\):/ ||
 	/Block in use \(biu\):/ {
 		in_mel = 1
-		stack_string=""
+		stack_string = ""
 		next
 	}
 	in_mel == 1 && /^$/ {
@@ -526,7 +633,7 @@ parse_logfile_dbx()
 		next
 	}
 	in_mel == 1 {
-		mel_line += 1;
+		mel_line += 1
 	}
 	/Found leaked block of size/ {
 		bytes += $6
@@ -549,8 +656,8 @@ parse_logfile_dbx()
 		next
 	}
 	mel_line > 2 {
-		gsub(/\(\)/,"")
-		new_line = $2;
+		gsub(/\(\)/, "")
+		new_line = $2
 		stack_string = "/" new_line stack_string
 		next
 	}
@@ -560,17 +667,23 @@ parse_logfile_dbx()
 	}
 	/execution completed/ {
 		runs += 1
+		lbytes += bytes
+		minbytes = (minbytes < bytes) ? minbytes : bytes
+		maxbytes = (maxbytes > bytes) ? maxbytes : bytes
+		bytes = 0
+		lblocks += blocks
+		minblocks = (minblocks < blocks) ? minblocks : blocks
+		maxblocks = (maxblocks > blocks) ? maxblocks : blocks
+		blocks = 0
 		next
 	}
 	END {
-		print "# " bytes " bytes " blocks " blocks in " runs " runs" > "/dev/stderr";
+		print "# " lbytes " bytes " lblocks " blocks in " runs " runs " \
+		minbytes " minbytes " maxbytes " maxbytes " minblocks " minblocks " \
+		maxblocks " maxblocks " > "/dev/stderr"
 	}' 2> ${TMP_COUNT}
 	
-	read hash lbytes bytes_str lblocks blocks_str in_str lruns rest < ${TMP_COUNT}
-	tbytes=`expr "${tbytes}" + "${lbytes}"`
-	tblocks=`expr "${tblocks}" + "${lblocks}"`
-	truns=`expr "${truns}" + "${lruns}"`
-	rm ${TMP_COUNT}
+	stat_add
 }
 
 ######################## parse_logfile_valgrind ########################
@@ -579,68 +692,82 @@ parse_logfile_dbx()
 parse_logfile_valgrind()
 {
 	${AWK} '
-	 BEGIN {
-		 in_mel=0;
-		 in_sum=0;
-		 bytes=0;
-		 blocks=0;
-		 runs=0;
-		 stack_string="";
-		 bin_name=""; }
-	 !/==[0-9]*==/ { 
-		if ( $1 == "'${VALGRIND}'" ) 
-			bin_name = $8 ; 
-		next;
-	 }
-	 /blocks are/ {
-		in_mel = 1;
-		stack_string="";
-		next;
-	 }
-	 /LEAK SUMMARY/ {
-		in_sum=1;
-		runs += 1;
-		next;
-	 }
-	 /^==[0-9]*== *$/ { 
-	    	if (in_mel)
-			print bin_name stack_string;
-		in_sum = 0;
-		in_mel = 0;
-		next;
-	 }
-	 in_mel == 1 {	
-		new_line = $4;
+	BEGIN {
+		in_mel = 0
+		in_sum = 0
+		bytes = 0
+		lbytes = 0
+		minbytes = 9999999
+		maxbytes = 0
+		blocks = 0
+		lblocks = 0
+		minblocks = 9999999
+		maxblocks = 0
+		runs = 0
+		stack_string = ""
+		bin_name = "" 
+	}
+	!/==[0-9]*==/ { 
+		if ( $1 == "Running:" ) 
+			bin_name = $2
+			bin_nf = split(bin_name, bin_fields, "/")
+			bin_name = bin_fields[bin_nf]
+		next
+	}
+	/blocks are/ {
+		in_mel = 1
+		stack_string = ""
+		next
+	}
+	/LEAK SUMMARY/ {
+		in_sum = 1
+		next
+	}
+	/^==[0-9]*== *$/ { 
+		if (in_mel)
+			print bin_name stack_string
+		if (in_sum) {
+			runs += 1
+			lbytes += bytes
+			minbytes = (minbytes < bytes) ? minbytes : bytes
+			maxbytes = (maxbytes > bytes) ? maxbytes : bytes
+			bytes = 0
+			lblocks += blocks
+			minblocks = (minblocks < blocks) ? minblocks : blocks
+			maxblocks = (maxblocks > blocks) ? maxblocks : blocks
+			blocks = 0
+		}
+		in_sum = 0
+		in_mel = 0
+		next
+	}
+	in_mel == 1 {	
+		new_line = $4
 		if ( new_line == "(within")
-			new_line = "*";
-		stack_string = "/" new_line stack_string;
-	 }
-	 in_sum == 1 {
-		for (i=2; i <= NF; i++) {
+			new_line = "*"
+		stack_string = "/" new_line stack_string
+	}
+	in_sum == 1 {
+		for (i = 2; i <= NF; i++) {
 			if ($i == "bytes") {
-				str = $(i-1);
-				gsub(",","",str);
-				bytes += str;
+				str = $(i - 1)
+				gsub(",", "", str)
+				bytes += str
 			}
 			if ($i == "blocks.") {
-				str = $(i-1);
-				gsub(",","",str);
-				blocks += str;
+				str = $(i - 1)
+				gsub(",", "", str)
+				blocks += str
 			}
 		}
-	 }
-	 END {
-		print "# " bytes " bytes " blocks " blocks in " runs " runs" > "/dev/stderr";
-	 }' 2> ${TMP_COUNT}
-
-	# sigh it would be nice to just pipe stderr and let stdout go by I've never been 
-	# able to convince any shell to do that correctly, so we are reduced to using a temp
-	# file
-	read hash lbytes bytes_str lblocks blocks_str in_str lruns rest < ${TMP_COUNT}
-	tbytes=`expr "${tbytes}" + "${lbytes}"`
-	tblocks=`expr "${tblocks}" + "${lblocks}"`
-	truns=`expr "${truns}" + "${lruns}"`
-	rm ${TMP_COUNT}
+	}
+	END {
+		print "# " lbytes " bytes " lblocks " blocks in " runs " runs " \
+		minbytes " minbytes " maxbytes " maxbytes " minblocks " minblocks " \
+		maxblocks " maxblocks " > "/dev/stderr"
+	}' 2> ${TMP_COUNT}
+	
+	stat_add
 }
 
 ############################# check_ignored ############################
@@ -648,7 +775,7 @@ parse_logfile_valgrind()
 ########################################################################
 check_ignored()
 {
-    ${AWK} -F/ '
+	${AWK} -F/ '
 	BEGIN {
 		ignore = "'${IGNORED_STACKS}'"
 		# read in the ignore file
@@ -664,25 +791,25 @@ check_ignored()
 				continue
 			} else {
 				bugnum_array[count] = BUGNUM
- 				# Create a regular expression for the ignored stack:
- 				# replace * with % so we can later replace them with regular expressions
- 				# without messing up everything (the regular expressions contain *)
- 				gsub("\\*","%",line)
- 				# replace %% with .*
- 				gsub("%%",".*",line)
- 				# replace % with [^/]*
- 				gsub("%","[^/]*",line)
- 				# add ^ at the beginning
- 				# add $ at the end
- 				line_array[count] = "^" line "$"
+				# Create a regular expression for the ignored stack:
+				# replace * with % so we can later replace them with regular expressions
+				# without messing up everything (the regular expressions contain *)
+				gsub("\\*", "%", line)
+				# replace %% with .*
+				gsub("%%", ".*", line)
+				# replace % with [^/]*
+				gsub("%", "[^/]*", line)
+				# add ^ at the beginning
+				# add $ at the end
+				line_array[count] = "^" line "$"
 				count++
 			}
 		}
 	}
 	{
-		match_found=0
+		match_found = 0
 		# Look for matching ignored stack
-	 	for (i=0 ; i < count; i++) {
+		for (i = 0; i < count; i++) {
 			if ($0 ~ line_array[i]) {
 				# found a match
 				match_found = 1
@@ -714,13 +841,15 @@ check_ignored()
 ########################################################################
 log_parse()
 {
-	echo "${SCRIPTNAME}: Processing log ${LOGNAME}:"
 	${PARSE_LOGFILE} < ${LOGFILE} > ${TMP_STACKS}
-	cat ${TMP_STACKS} | sort -u | check_ignored
+	echo "${SCRIPTNAME}: Processing log ${LOGNAME}:" > ${TMP_SORTED}
+	cat ${TMP_STACKS} | sort -u | check_ignored >> ${TMP_SORTED}
 	ret=$?
-	rm ${TMP_STACKS}
-	echo ""
-
+	echo >> ${TMP_SORTED}
+	
+	cat ${TMP_SORTED} | tee -a ${FOUNDLEAKS}
+	rm ${TMP_STACKS} ${TMP_SORTED}
+	
 	return ${ret}
 }
 
@@ -736,10 +865,69 @@ cnt_total()
 	echo ""
 }
 
+############################### run_ocsp ###############################
+# local shell function to run ocsp tests
+########################################################################
+run_ocsp()
+{
+	stat_clear
+	
+	cd ${QADIR}/iopr
+	. ./ocsp_iopr.sh
+	ocsp_iopr_run
+	
+	stat_print "Ocspclnt"
+}
+
+############################## run_chains ##############################
+# local shell function to run PKIX certificate chains tests
+########################################################################
+run_chains()
+{
+    stat_clear
+
+    LOGNAME="chains"
+    LOGFILE=${LOGDIR}/chains.log
+
+    . ${QADIR}/chains/chains.sh
+
+    stat_print "Chains"
+}
+
+############################## run_chains ##############################
+# local shell function to run memory leak tests
+#
+# NSS_MEMLEAK_TESTS - list of tests to run, if not defined before,
+# then is redefined to default list 
+########################################################################
+memleak_run_tests()
+{
+    nss_memleak_tests="ssl_server ssl_client chains ocsp"
+    NSS_MEMLEAK_TESTS="${NSS_MEMLEAK_TESTS:-$nss_memleak_tests}"
+
+    for MEMLEAK_TEST in ${NSS_MEMLEAK_TESTS}
+    do
+        case "${MEMLEAK_TEST}" in
+        "ssl_server")
+            run_ciphers_server
+            ;;
+        "ssl_client")
+            run_ciphers_client
+            ;;
+        "chains")
+            run_chains
+            ;;
+        "ocsp")
+            run_ocsp
+            ;;
+        esac
+    done
+}
+
 ################################# main #################################
 
 memleak_init
-run_ciphers_server
-run_ciphers_client
+memleak_run_tests
 cnt_total
 memleak_cleanup
+
